const PrefixesListSingleton = (function() {
    var instance = null;

    function PrefixesList() {
        this.prefixesList = null;
        this.initPrefixesArray = function() {
            dumpStr('-> initPrefixesArray');
            var dataString;
            var items = new Array();

            try {
                dataString = rdi_prefs.getComplexValue("rds", Components.interfaces.nsISupportsString).data;

                if (dataString != "") {
                    if (dataString.indexOf(getRDEntrySplitSign()) < 0) {
                        dataString = upgradeSettings(dataString);
                    }
                    var itemStrings = dataString.split(getRDPrefEntriesSplitSign());
                    
                    for (var i = 0; i < itemStrings.length; i ++) {
                        var itemData = itemStrings[i].split(getRDEntrySplitSign());

                        var item = new PrefixItem(itemData.shift(), itemData.shift());
                        item.aliases = itemData;
                        items.push(item);
                    }
                }
                this.currentDataString = dataString;
            } catch (e) {
                dumpStr(e);
            }

            items.indexOf = function(elt) {
                for (var i = 0; i < this.length; i++) {
                    if (this[i].equals(elt)) {
                        return i;
                    }
                }
                return -1;
            };

            items.removeAll = function() {
                for (var i = this.length - 1; i >= 0; i--) {
                    this.pop();
                }
            };

            dumpStr('<- initPrefixesArray; '+items);
            return items;
        };
        this.savePrefixesArray = function (defaultsignature) {
            dumpStr('-> savePrefixesArray');
            var writer = {
                sb:       [],
                writeItem:    function (s) {
                    var entry;
                    entry = s.description + getRDEntrySplitSign() + s.rd;
                    if (s.aliases && s.aliases.length > 0) {
                         entry += getRDEntrySplitSign() + s.aliases.join(getRDEntrySplitSign());
                    }
                    this.sb.push(entry);
                },
                toString: function () {  return this.sb.join(getRDPrefEntriesSplitSign()); }
            };

            this.prefixesList.forEach(writer.writeItem, writer);
            str.data = writer.toString();

            rdi_prefs.setComplexValue("rds", Components.interfaces.nsISupportsString, str);
            if (defaultsignature != undefined) {
                rdi_prefs.setCharPref("defaultrd", defaultsignature);
            }

            dumpStr('<- savePrefixesArray; '+str.data);
        };
        this.getPrefixesList = function () {
            var dataString = rdi_prefs.getComplexValue("rds", Components.interfaces.nsISupportsString).data;
            if (this.currentDataString != dataString) {
                this.prefixesList.removeAll();
                var newList = this.initPrefixesArray();
                newList.forEach(function(s) { PrefixesListSingleton.getInstance().prefixesList.push(s) });
            }

            return this.prefixesList;
        }
        this.prefixesList = this.initPrefixesArray();
    }

    return new function() {
        this.getInstance = function() {
            if (instance == null) {
                instance = new PrefixesList();
                instance.constructor = null;
            }
            return instance;
        }
    }
})();

function PrefixItem(aLabel, aPrefix) {
    this.label = aLabel;
    this.prefix = aPrefix;
    this.aliasesList = [];
}

PrefixItem.prototype = {
    isValid : function() {
        dumpStr("isValid "+this.prefix+" "+this.label);
        return (this.prefix != "" && this.label != "");
    },

    get description() { return this.label;  },
    get rd() {          return this.prefix; },
    get aliases() {     return this.aliasesList; },

    set description(aLabel) { this.label  = aLabel;  },
    set rd(aPrefix) {         this.prefix = aPrefix; },
    set aliases(aAliases)   { this.aliasesList = aAliases; },

    toString : function()   { return ('[' + this.description + getRDEntrySplitSign() + this.rd + ']'); },

    equals : function (otherItem) {
        dumpStr('-> equals; this:'+this+'; other:'+otherItem);
        if (compare(this.rd, otherItem.rd))
            return true;

        for (var i = 0; i < this.aliases.length; i++) {
            if (compare(this.aliases[i], otherItem.rd)) {
                return true;
            }
            for (var j = 0; i < otherItem.aliases.length; j++) {
                if (compare(this.rd, otherItem.aliases[j])) {
                    return true;
                }
                if (compare(this.aliases[i], otherItem.aliases[j])) {
                    return true;
                }
            }
        }

        return false;
    }
}

function compare(sb1, sb2) {
    return (removeIgnoredSigns(sb1).toLowerCase().trim()
                == removeIgnoredSigns(sb2).toLowerCase().trim());
}

function removeIgnoredSigns(sb) {
    let ignoreSigns = rdi_prefs.getCharPref("discoveryIgnoreSigns");
    for (var i = 0; i < ignoreSigns.length; i++) {
        sb = sb.split(ignoreSigns.charAt(i)).join('');
    }
    sb = sb.split(' ').join('');
    return sb;
}


